/*
 *  ====================================================================================================================================================================
 *
 *  Application : Code guru submission
 *
 *  Module .... : core.cpp
 *  Author .... : Tony Johnson & Chris Wilcock
 *  Purpose ... : Demonstrates using the Standard Microsoft Data Link Dialog.
 *
 *  ===================================================================================================================================================================
 *
 */
#include <iostream.h>
#include <fstream.h>
#include <comdef.h>


 
/*
 *   Lets use the import directive to create some smart pointers for us
 */
#import "c:\program files\common files\system\ado\msado20.tlb" no_namespace rename("EOF", "IsEOF") rename("BOF", "IsBOF")
#import "c:\program files\common files\system\ole db\oledb32.dll" rename_namespace("oledb")


/*
 *  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
 *
 *  Function ..... : inline void TESTHR( HRESULT hr )
 *  Purpose ...... : Inline HRESULT tester function that triggers a _com_error exception for us
 *  Parameters ... : HRESULT hr - The result to test for failure
 *  Returns ...... : Nothing
 *
 */
inline void TESTHR( HRESULT hr )
	{
	if( FAILED(hr) ) _com_issue_error( hr );
	}	

/*
 *  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
 *
 *  Function Prototypes
 *
 */
BOOL	ShowDataLink( _bstr_t* );


/*
 *  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
 *
 *  Function ..... : void main( )
 *  Purpose ...... : The program entry point
 *  Parameters ... : None
 *  Returns ...... : Nothing
 *
 */
void main( )
	{

	fstream		iosCfg;
	char		lpsz_Buffer[ 512 ];
	_bstr_t		bstr_myConnectString="";

	/*
	 *   Initialise COM
	 */
	if( FAILED(CoInitialize( NULL )) ) 
		{
		cout << "\nFailed to initialise COM";
		return;
		}

	/*
	 *   Clean up the temporary buffer that will be used to store the connection string
	 */
	memset( lpsz_Buffer, 0, 512 );

	/*
	 *	 Open an input stream to see if a connection string is already defined.
	 */
	iosCfg.open( "my.cfg", ios::in|ios::nocreate );
	
	if( !iosCfg.fail() )
		{
		/*
		 *   The file my.cfg was opened with success, read the connection string 
		 *   into the temp buffer and copy this into the _bstr_t 
		 */
		iosCfg.getline( lpsz_Buffer, 512 );
		bstr_myConnectString = lpsz_Buffer;
		}
	
	/*
	 *   Close the input stream
	 */
	iosCfg.close();
	

	/* 
	 *   Show the data link dialog, using the connection string we have ( may still be blank )
	 *   
	 *   If this function returns TRUE, we need to store the connect string for retrieval later
	 */
	if( ShowDataLink( &bstr_myConnectString ) )
		{
		/*
		 *   We are storing the connection string. 
		 *	 Clear the state flags, open the file for output and write to it
		 */
		iosCfg.clear();
		iosCfg.open( "my.cfg", ios::out );
		
		if( !iosCfg.fail() )
			{
			iosCfg << (char*) bstr_myConnectString << "\n";
			}
		
		iosCfg.close();
		}

	/*
	 *   We are done, so uninit COM
	 */
	CoUninitialize();
	}


/*
 *  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
 *
 *  Function ..... : BOOL ShowDataLink( _bstr_t * bstr_ConnectString )
 *  Purpose ...... : Display the OLE DB Data Link properties dialog, using (if supplied) a 
 *				   : pre-configured connection string.
 *  Parameters ... : bstr_ConnectString = A pointer to a _bstr_t that currently holds the 
 *				   :                      connection string. Upon completion, it will contain 
 *				   :					  the new connection string.
 *  Returns ...... : TRUE indicating OK was pressed to save the connection string.
 *
 */
BOOL ShowDataLink( _bstr_t * bstr_ConnectString )
	{

	HRESULT		hr;
	oledb::IDataSourceLocatorPtr	p_IDSL= NULL;			// This is the Data Link dialog object
	_ConnectionPtr					p_conn = NULL;			// We need a connection pointer too
	BOOL							b_ConnValid = FALSE;
	
	try
		{
	
		/*
		 *   Create an instance of the IDataSourceLocator interface and check for errors.
		 */
		hr = p_IDSL.CreateInstance( __uuidof( oledb::DataLinks ));
		TESTHR( hr );

		/*
		 *   If the passed in Connection String is blank, we are creating a new connection
		 */
		if( *bstr_ConnectString == _bstr_t("") )
			{
			/*
			 *   Simply display the dialog. It will return a NULL or a valid connection object 
			 *   with the connection string filled in for you. If it returns NULL, the user
			 *	 must of clicked the cancel button.
			 */
			p_conn = p_IDSL->PromptNew();
			if( p_conn != NULL ) b_ConnValid = TRUE;
			
			}
		else 
			{
			/*
			 *   We are going to use a pre-existing connection string, so first we need to
			 *	 create a connection object ourselves. After creating it, we'll copy the
			 *   connection string into the object's connection string member
			 */
			p_conn.CreateInstance( "ADODB.Connection" );
			p_conn->ConnectionString = *bstr_ConnectString;

			/*	 
			 *   When editing a data link, the IDataSourceLocator interface requires we pass 
			 *	 it a connection object under the guise of an IDispatch interface. So,
			 *	 here we'll query the interface to get a pointer to the IDispatch interface,
			 *	 and we'll pass that to the data link dialog.
			 */
			IDispatch * p_Dispatch = NULL;
			p_conn.QueryInterface( IID_IDispatch, (LPVOID*) & p_Dispatch );
			
			if( p_IDSL->PromptEdit( &p_Dispatch )) b_ConnValid = TRUE;
			
			p_Dispatch->Release();
			}
		/*   
		 *   Once we arrive here, we're done editing. Did the user press OK or cancel ?
		 *
		 *	 If OK was pressed, we want to return the new connection string in the parameter given to us.
		 */
		if( b_ConnValid )
			{
			*bstr_ConnectString = p_conn->ConnectionString;
			}

		}
	catch( _com_error & e )
		{
		
		/*
		 *   Catcher for any _com_error's that are generated.
		 */
		
		cout << "\nCom Exception raised\n";
		cout << "Description : " << (char*) e.Description() << "\n";
		cout << "Message     : " << (char*) e.ErrorMessage() << "\n";		
		
		return FALSE;
		}

	return b_ConnValid;
	}



/*
 *  ====================================================================================================================================================================
 *
 *  Eof : core.cpp
 *
 */
